#ifndef __EVENTHANDLER_H__
#define __EVENTHANDLER_H__


#define HASH(x,y) hash_safe(x,y)


/****************************************/
/**
  * Socket/Select Events
  */



enum EventType {

    EVENT_READ =  0,
    EVENT_WRITE = 1,
    EVENT_ERROR = 2,
    EVENT_CONNECT = 3,
    EVENT_ENDBURST = 4,
    EVENT_NICK = 5,
    EVENT_QUIT = 6,
    EVENT_VERSION = 7
};


typedef void cmd_fnct(int, char**);


typedef struct servcmd_ {

	char name[40];

        char* module;

	void (*fnct)(Link*,int,char**);

	struct servcmd_  *next,*prev;

} ServCmds;



typedef struct usercmd_ {

       char* module;

       int active;      //if our command is active

	void (*routine)(User*,int,char**);

	struct usercmd_  *next,*prev;

} UserCmds;


//event for PRIVMSG's/NOTICES
typedef struct modmessage_ {

	char name[10];

	void (*fnct) (char *src,char *dest,char *msg);

	struct modmessage_ *next,*prev;

} ModuleMessage;



typedef struct sockevent_ {

       char		name[25]; //used for identification.
       void		(*fnct)(Socket *, int);

} SockEvent;


typedef struct
{
	Channel		*c;
	User		*u;
} evh_channel;


/********************************************/



dlink_list UserCmdTable[1024];
dlink_list ServCmdTable[1024];
dlink_list sockevents;
dlink_list err_handlers;


/*********************************************/

typedef void HandleEvent (int);



/*********************************************/
/**
 * Server Events
 */

typedef struct EventBase
{
    time_t  age;

    char *module; //module the event is associated with.

    char name[32];

    int (*handler)(int,void *);

    int recall; //if the event can only be called once :)

    int called; //how many times each event was called.

    int error; //used for our catchCoreErr()

    time_t  when;
    int     interval;

} Events;

/****************************************/

HandleEvent* EventList[1024];

void AddSockEvent       (char *name, void (*fnct)(Socket*,int));
void HandleSockEvent    (Socket*, int);
void DelSockEvent       (char*);

void DelEvent(char *event, void* fnct);

void AddEvent	(char *, void*);
void Event		(char *, int, void *);


void AddServCmd       (char *,void (*fnct)(Link*,int,char**));
int  HandleServCmd    (char *, Link*, int , char **);
void DelServCmd        ();


ServCmds* FindServCmd (char *cmd);

//error stuffz
void catchCoreErr (int errcode, char *errstr);
void AddErrHandler (int errcode, void *fnct);

void AddUserCmd       (char *, void (*fnct)(User*,int,char**));
int  HandleUserCmd    (char *, User*, int , char **);
void DelUserCmd       (char *,void (*fnct)(User*,int,char**));

/************************************/

void HandleMsg       (char *msgtype,char *source, char *dest, char *msg);
void AddModuleMessage(char *msg, void (*fnct) (char *src,char *dest,char *msg));

dlink_list  events;

void AddTimedEvent (char *, void *, int);
void DelTimedEvent (char *);
void RunTimedEvents ();

void ev_housekeeping	(int, char **);
void ev_connectuplink	(int, char **);
void ev_ping			(int, char **);
#endif


