/*
 *         	  OMEGA IRC SECURITY SERVICES
 * 	      	    (C) 2008 Omega Dev Team
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *    $Id: module.h 1419 2008-09-15 08:53:08Z twitch $
 */

#ifndef __MODULE_H__
#define __MODULE_H__

//These are custome error strings.
#define MOD_ERR_DL -2
#define MOD_ERR_MSG -1

//Module isnt in error state.
#define MOD_ERR_OK 0

//core defined Error Strings.
#define MOD_ERR_STOP 1
#define MOD_ERR_CRIT 2
#define MOD_ERR_FATAL 3
#define MOD_ERR_EXISTS 4
#define MOD_ERR_NOFILE 5
#define MOD_ERR_IO 6
#define MOD_ERR_API 7
#define MOD_ERR_ABI 8
#define MOD_ERR_MEMORY 9
#define MOD_ERR_UNLOAD 10
#define MOD_ERR_SYNTAX 11
#define MOD_ERR_UNSAFE 12
#define MOD_ERR_NOMOD  13
#define MOD_ERR_SQL    14



//Quick referance module errors... designed for ease of use.
#define MOD_CONT MOD_ERR_OK
#define MOD_STOP MOD_ERR_STOP
#define MOD_CRIT MOD_ERR_CRIT
#define MOD_FATAL MOD_ERR_FATAL
#define MOD_MEMORY MOD_ERR_MEMORY


//Module type definations
#define MOD_TYPE_UNKNOWN  0  //unknown or loaded manually after startup
#define MOD_TYPE_PROTOCOL 1  //protocol module
#define MOD_TYPE_CORE     2  //core modules
#define MOD_TYPE_STD      3  //standard modules
#define MOD_TYPE_CLIENT   4  //client module.

char mod_err_msg[512];


#define throwModErr(msg, ...) \
		snprintf(mod_err_msg, sizeof(mod_err_msg), msg, __VA_ARGS__); \
		return MOD_ERR_MSG; \


/**
 * This just looks ugly but i wanted to take into account
 * for directories not having a / and having a /
 * So with this said :) this is what i came up with - Twitch
 */

#define format_filename(dest,filestr)  \
    snprintf(dest,sizeof(dest) - 1,  "%s",(*(filestr) != '/')? find_module_dir((filestr)) : ""); \
    snprintf(dest, sizeof(dest) - 1, "%s%s%s%s",(dest), (dest[strlen((dest))] != '/' && *(filestr) != '/')? "/" : "",  \
            (filestr), (filestr[strlen((filestr)) - 3] != '.')? ".so" : "");

/**
  * Module ABI Stuff
  *   This logic kinda looks off at first glance however we need this to make sure
  *   that module API's match, note you can manually change the ABI version on a
  *   module however do that at your own risk, as the module ABI specifies which
  *   modules the core will work properly with.
  */

#define ABI_MAJOR  0
#define ABI_MINOR  5
#define ABI_PATCH  2

#define ABI_VERSION ((ABI_MAJOR * 1000) + (ABI_MINOR * 100) + ABI_PATCH)
#define MAKE_ABI(maj,min,patch) ((maj * 1000) + (min * 100) + patch)

/**
  *   Module Header
  *   	This struct allows the core to pull in the nessicary information
  *   	and place it into the module structure.
  *
  *  @param name - The module name -> This is just for personal referance
  *                          as the core relies on aboslute paths to find and remove
  *                         modules.
  *  @param version - The modules version in free form
  *  @param abi      - The Binary interface version, this is the version of the API
  *                            used to interact with this module.
  *  @param load     - The function called by the core when loading the module
  *  @param unload  - The function used when unloading a module from the core.
  *
  */

#define MODHEADER(name,version,author,abi,load,unload) \
ModuleInfo ModInfo = {          \
    name, version, author, abi, \
    load, unload }


int nomodules;


/************************************************/

/**
 * ModInfo - This structure holds module specific information, this is information
 * 		 that is not held to be true at all times, since the modules interact with it
 *                 given that it is only used for basic checks :)
 *
 *  @param name - The module name -> This is just for personal referance
 *                          as the core relies on aboslute paths to find and remove
 *                         modules.
 *  @param version - The modules version in free form
 *  @param abi      - The Binary interface version, this is the version of the API
 *                            used to interact with this module.
 *  @param mod_register   - The function called by the core when loading the module,
 *				    and takes place in the registeration phase of the load.
 *
 *	NOTE: this is a required function - Modules will fail to load  if this is not specified.
 *
 *  @param mod_unregister  - The function used when unloading a module from the core,
 *				      takes place before the dlclose function is called, in the un-register phase
 *
 *	NOTE: This function is not required modules, however will not unload if this is not specified.
 *
 */

typedef struct moduleinfo_ {
     char *name;        //name of module (Modules are allowed to specify there own name)

     char *version;     //Version information in a string - This has no impact on weather or

     char *author;      //Who wrote the module

     long abi;          //BINARY INTERFACE version

     int      (*mod_register)	(void);	    /* Register function;   */

	 void	  (*mod_unregister)	(void);		/* Unregister function.	*/


} ModuleInfo;


/**
 *     Module structures (Module)
 *       This holds primary module information, such as the handle, pathway, and time the module was loaded.
 *	     this information is to be held true and trusted at all times, as the MODULE system is the only thing
 *	     that is allowed to touch this information :)
 */

typedef struct module_ {

    time_t age;            //Time we were loaded

    char file[256];         //We have to store the filename because all load/unload functions work off paths.
                            //also this keeps a standard between them. :)

    void* handle;

    int type;               //Module types

    ModuleInfo *mi;

} Module;


dlink_list  modules;

/************************************************/
/**
 * Module Que Structures
 */


typedef struct moduleq_entry {
	char *name;
	int action;
} ModuleQEntry;

dlink_list moduleque;


/***********************************/



int load_protocol();
int load_core();
int load_modules();
int load_client_modules();

int rehash_core_modules();
int rehash_client_modules();
void unload_all_modules();

/*************************************/


int     module_open   (char *, int);
Module* module_find   (char *);
int     module_exists (char *);
void    module_free   (Module *);


char *find_module_dir(char *module);


int addModuleQue(char *,int);
int runModuleQue();

int RegisterModule(char*, char* ,char *, int, int ,int);
int CheckVer();


#endif /*__MODULE_H__*/


