#include "stdinc.h"
#include "server.h"
#include "extern.h"

#define NAME	"g_dcc"
#define VERSION "$Id: g_dcc.c 1423 2008-09-16 07:45:54Z twitch $"
#define AUTHOR	"Jer"

#define MAX_PARA	15

static void	g_privmsg		(User *, int, char **);
static void g_chat			(User *, char *);

static void dcc_sockhandle	(Socket *, int);
static void dcc_estab		(Socket *);
static void	dcc_accept		(Socket *);
static void dcc_read		(Socket *);

static void dcc_parse		(Socket *, char *);

static void sendto_all_dcc	(Socket *, char *, ...);

static void dcc_cmd_userlist	(Socket *, int, char **);

static int Module_Init();
static void Module_Close();


MODHEADER(NAME, VERSION, AUTHOR, MAKE_ABI(0,5,2),Module_Init, Module_Close);



static int Module_Init()
{
	int ch;

	AddUserCmd	("PRIVMSG",	g_privmsg);
	AddCmd (s_Guardian, "CHAT", 0, g_chat);

	return MOD_CONT;

}

static void Module_Close()
{
	dlink_node	*dl, *tdl;
	Socket	*s;

	DLINK_FOREACH_SAFE (dl, tdl, sockets.head)
	{
		s	= dl->data;

		if (!(s->flags & SOCK_DCC))
			continue;

		sendto_socket (s, "DCC Module is being unloaded, closing session");
		close (s->sd);
		dlink_delete (dl, &sockets);
		dlink_free (dl);
		free (s);
	}

	DelUserCmd ("PRIVMSG", g_privmsg);

	DelCmd (s_Guardian, "CHAT", g_chat);

    return;
}

static void g_chat (User *source, char *msg)
{
	Socket	*sock;
	int				optval, flags, dcc_port, res;
	unsigned long	l_ip;
	struct hostent	*local_host;
	dlink_node		*dl;
	struct sockaddr_in	sa;

	if (CfgSettings.local_host == NULL)
	{
		sendto_one (s_Guardian, source, "DCC Failed.");
		sendto_logchan ("\002DCC\002: DCC Failed, local ip unknown "
							"Try setting localip in the conf file");
		return;
	}

	sock	= new_socket ();
	add_sock_handlers(sock, (void *) dcc_sockhandle, NULL);

	if ((sock->sd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
	{
		dl	= dlink_find_delete (sock, &sockets);
		dlink_free (dl);
		free (sock);

		return;
	}

	if (sock->sd > maxfds)
		maxfds	= sock->sd;

	optval	= 1;
	setsockopt (sock->sd, SOL_SOCKET, SO_REUSEADDR, (char *) &optval, sizeof(optval));

	for (dcc_port = 3000; dcc_port < 3500; dcc_port++)
	{
		memset ((void *) &sa, '\0', sizeof(sa));
		sa.sin_family		= AF_INET;
		sa.sin_addr.s_addr	= INADDR_ANY;
		sa.sin_port			= htons(dcc_port);

		if ((res = bind (sock->sd, (struct sockaddr *) &sa, sizeof(sa)) >= 0))
			break;
	}

	if (res < 0)
	{
		close (sock->sd);
		dl = dlink_find_delete (sock, &sockets);
		dlink_free (dl);
		free (sock);
		sendto_logchan ("\002DCC\002: Unable to bind to DCC port");
		return;
	}

	flags	= fcntl (sock->sd, F_GETFL, 0);
	flags	|= O_NONBLOCK;
	(void) fcntl (sock->sd, F_SETFL, flags);

	if (listen (sock->sd, 1) < 0)
	{
		close (sock->sd);
		dl = dlink_find_delete (sock, &sockets);
		dlink_free (dl);
		free (sock);

		sendto_logchan ("\002DCC\002: Unable to listen()");
		return;
	}

	sock->flags |= SOCK_DCCIN|SOCK_CONNECTING|SOCK_DCC;
//	sock->flags	|= SOCK_READ|SOCK_DCCCONN|SOCK_DCC|SOCK_NOREAD;
	sock->user	= source;

	local_host	= CfgSettings.local_host;
	(void) memcpy ((void *) &l_ip, (void *) local_host->h_addr, sizeof(local_host->h_addr));

	l_ip	= htonl (l_ip);

	sendto_one_privmsg (s_Guardian, source, "\001DCC CHAT chat %lu %d\001", l_ip, dcc_port);
	return;
}

static void g_privmsg	(User *source, int argc, char **argv)
{
	int		i, cnt, port, flags, optval, ret;
	char	*params[10];
	Socket	*sock;

	struct sockaddr_in	sa;
	unsigned long		hl;
	dlink_node			*dl;

#ifdef HAVE_GETADDRINFO
	struct addrinfo	*hostres;
#endif

	if (!source)
		return;

	if ((*argv[2] != '\001') || (*argv[1] == '#'))
		return;

	if (argc < 3)
		return;

	if ((strcasecmp (argv[2], "\001DCC")) == 0)
		return;

	cnt	=	generictoken (' ', 10, argv[2], params);

	if (cnt < 5)
		return;

	if ((strcasecmp ("CHAT", params[1])) == 0)
	{
		port	= atol(params[4]);

		if (cnt < 5)
			return;

		if ((port < 1024) || (port > 65535))
		{
			sendto_logchan ("\002DCC\002: Invalid port");
			printf ("Port is out of range\n");
			return;
		}

		sock	= new_socket (NULL, 0, NULL, NULL);
		hl		= strtoul (params[3], NULL, 10);
		optval	= 1;

#ifdef HAVE_GETADDRINFO
		if (strstr(params[3], ":"))
		{
			hostres	= gethostinfo (params[3], port);

			if ((sock->sd = socket (hostres->ai_family, hostres->ai_socktype, hostres->ai_protocol)) < 0)
			{
				dl	= dlink_find_delete (sock, &sockets);
				dlink_free (dl);
				free (sock);

				return;
			}

			setsockopt (sock->sd, SOL_SOCKET, SO_REUSEADDR, (char *) &optval, sizeof(optval));

        	flags   = fcntl (sock->sd, F_GETFL, 0);
        	flags   |= O_NONBLOCK;
		    (void) fcntl (sock->sd, F_SETFL, flags);

			connect (sock->sd, hostres->ai_addr, hostres->ai_addrlen);

			sock->flags	|= SOCK_WRITE|SOCK_DCC;

			if (sock->sd > maxfds)
				maxfds	= sock->sd;

			return;
		}
#endif

		if ((sock->sd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
		{
			/* Remove the socket from our list */
			dl = dlink_find_delete (sock, &sockets);
			dlink_free (dl);
			free (sock);

			printf ("Unable to open socket\n");
			return;
		}

		printf ("Connecting to %lu port %d\n", hl, port);

		setsockopt (sock->sd, SOL_SOCKET, SO_REUSEADDR, (char *) &optval, sizeof(optval));

		memset ((void *) &sa, '\0', sizeof(sa));
		sa.sin_family		= AF_INET;	/* Maybe IPv6 DCC in the future? */
		sa.sin_port			= htons(port);
		sa.sin_addr.s_addr	= htonl(hl);

		flags	= fcntl (sock->sd, F_GETFL, 0);
		flags	|= O_NONBLOCK;
		(void) fcntl (sock->sd, F_SETFL, flags);

		connect (sock->sd, (struct sockaddr *) &sa, sizeof(sa));
		sock->flags	|= SOCK_CONNECTING|SOCK_DCCOUT|SOCK_DCC;
		sock->user	= source;

		if (sock->sd > maxfds)
			maxfds	= sock->sd;

		return;
	}

	return;
}

static void dcc_sockhandle (Socket *sock, int e)
{
	if (!(sock->flags & SOCK_DCC))
		return;


	switch (e)
	{
		case EVENT_WRITE:
			if (sock->flags & SOCK_CONNECTING)
				if (sock->flags & SOCK_DCCOUT)
					dcc_estab (sock);
			break;

		case EVENT_READ:
			if (sock->flags & SOCK_CONNECTING)
			{
				if (sock->flags & SOCK_DCCIN)
					dcc_accept (sock);
				break;
			}
			else
				dcc_read (sock);
			break;
	}

	return;
}

void dcc_estab (Socket *s)
{
	int			ret;
	dlink_node	*dl;

	ret		= sendto_socket (s, "Welcome to %s, version Omega(SilverSky)-%s", CfgSettings.servername, VERSION_STRING_DOTTED);

	if (ret == 0)
	{
		sendto_logchan ("\002DCC\002: User connected");

		s->flags	&= ~SOCK_CONNECTING;
		s->flags	|= SOCK_DCC;
		return;
	}
	else
	{
		/* We were unable to connect so destroy the socket */
		sendto_logchan ("\002DCC\002: Unable to complete DCC connection: %s\n", strerror(errno));
		printf ("Unable to complete DCC connection: %s\n", strerror(errno));
		return;
	}

	return;
}

void dcc_accept (Socket *s)
{
	int		asock;
	int		addrlen = sizeof(struct sockaddr);
	struct sockaddr_in	*incoming_addr;
	dlink_node	*dl;

	incoming_addr = malloc(sizeof(struct sockaddr_in));

	if ((asock = accept(s->sd, (struct sockaddr *) incoming_addr, (socklen_t *) &addrlen)) < 0)
	{
		if (errno == EAGAIN)
			return;

		printf ("dcc_accept: accept() failed\n");

		shutdown (s->sd, SHUT_RDWR);
		close (s->sd);

		dl	= dlink_find_delete (s, &sockets);
		dlink_free (dl);
		free (s);
		free (incoming_addr);

		return;
	}

	if (asock > maxfds)
		maxfds = asock;

	shutdown (s->sd, SHUT_RDWR);
	close (s->sd);

	s->sd	= asock;
/*
	s->flags	&= ~SOCK_DCCCONN;
	s->flags	&= ~SOCK_NOREAD;
*/
	s->flags	&= ~SOCK_CONNECTING;
	s->sa	= incoming_addr;


	if (s->sa)
		sendto_logchan ("\002DCC\002: Client connected from %s", inet_ntoa(s->sa->sin_addr));

	sendto_socket (s, "Welcome");
	return;
}

static void dcc_read (Socket *source)
{
	char	*msg;

	while (source->fullLines > 0)
	{
		msg	= GetLine (source);

		if (msg == NULL)
		{
			printf ("No message\n");
			break;
		}

		printf ("msg: %s\n", msg);

		if (*msg != '.')
			sendto_all_dcc (source, msg);

		dcc_parse (source, msg);

		free (msg);
	}
	return;
}

static void dcc_parse (Socket *source, char *line)
{
	char	*argv[MAX_PARA];
	int		argc;

	memset (argv, '\0', sizeof(argv));

	argc	= generictoken (' ', MAX_PARA, line, argv);

	if (argv[0] == '\0')
		return;

	if (*argv[0] != '.')
		return;

	argv[0]++;

	if ((strcasecmp ("userlist", argv[0])) == 0)
		dcc_cmd_userlist (source, argc, argv);

	printf ("%s\n", argv[0]);

}

/**
 ** DCC Commands
 */

static void dcc_cmd_userlist	(Socket *source, int argc, char **argv)
{
	dlink_node	*dl;
	User		*u;
	int			n;

	n	= 0;

	DLINK_FOREACH (dl, userlist.head)
	{
		u	= dl->data;
		n++;

		sendto_socket (source, "[%d] %s (%s@%s) connected at %s", n, u->nick, u->user,
					u->host, u->serv->name);
	}

	return;
}

static void sendto_all_dcc (Socket *source, char *fmt, ...)
{
	dlink_node	*dl;
	Socket		*s;
	char		line[1024];
	va_list		args;

	va_start (args, fmt);
	vsnprintf (line, 1024, fmt, args);
	va_end (args);

	printf ("line: %s\n", line);

	DLINK_FOREACH (dl, sockets.head)
	{
		s	= dl->data;

		if (!(s->flags & SOCK_DCC))
			continue;

		sendto_socket (s, "[%s]: %s", (s->user)? source->user->nick : "Unknown", line);
	}
}
