#include "stdinc.h"
#include "server.h"
#include "extern.h"

#define NAME	"g_debug"
#define VERSION "$Id: g_debug.c 1415 2008-09-12 07:43:45Z twitch $"
#define AUTHOR	"Jer"

extern dlink_list servcmds;
extern dlink_list usercmds;

extern dlink_list ServCmdTable[1024];
extern dlink_list UserCmdTable[1024];


static void cmd_chaninfo	(User *, char *);
static void cmd_chanlist	(User *, char *);
static void cmd_userinfo	(User *, char *);
static void cmd_userlist	(User *, char *);
static void cmd_seventlist  (User *, char *);
static void cmd_sendq		(User *, char *);
static void cmd_servlist	(User *, char *);
static void cmd_servcmds    (User *, char *);
static void cmd_usercmds    (User *, char *);
static void cmd_ucmdinfo    (User *, char *);
static void cmd_eventlist   (User *, char *);
static void cmd_fdlist		(User *, char *);
static void cmd_progstats	(User *, char *);


static int Module_Init();
static void Module_Close();


MODHEADER(NAME, VERSION, AUTHOR, MAKE_ABI(0,5,2),Module_Init, Module_Close);

int Module_Init()
{
	int ch;

#ifdef DEBUG_MODE
	ch = AddCmd(s_Guardian, "CHANINFO",	ACC_FLAG_DEBUG,  cmd_chaninfo);
	ch = AddCmd(s_Guardian, "CHANLIST", 	ACC_FLAG_DEBUG,  cmd_chanlist);
	ch = AddCmd(s_Guardian, "USERINFO", 	ACC_FLAG_DEBUG,  cmd_userinfo);
	ch = AddCmd(s_Guardian, "USERLIST",	ACC_FLAG_DEBUG,  cmd_userlist);
	ch = AddCmd(s_Guardian, "SERVLIST", 	ACC_FLAG_DEBUG,  cmd_servlist);
   	ch = AddCmd(s_Guardian, "EVENTLIST",  	ACC_FLAG_DEBUG,  cmd_eventlist);
   	ch = AddCmd(s_Guardian, "SEVENTLIST", 	ACC_FLAG_DEBUG,  cmd_seventlist);
	ch = AddCmd(s_Guardian, "SENDQ",		ACC_FLAG_DEBUG,  cmd_sendq);
	ch = AddCmd(s_Guardian, "SERVCMDS", 	ACC_FLAG_DEBUG,  cmd_servcmds);
    	ch = AddCmd(s_Guardian, "CMDINFO", 	ACC_FLAG_DEBUG,  cmd_usercmds);
	ch = AddCmd(s_Guardian, "UCMDINFO", 	ACC_FLAG_DEBUG,  cmd_ucmdinfo);
	ch = AddCmd(s_Guardian, "FDLIST", 		ACC_FLAG_DEBUG,  cmd_fdlist);
	ch = AddCmd(s_Guardian, "PROGSTATS",      ACC_FLAG_DEBUG,  cmd_progstats);
#else
	throwModErr("This module requires debug to be enabled%s",".");
#endif
	return MOD_CONT;



}

void Module_Close()
{
         DelCmd(s_Guardian, "CHANINFO",  cmd_chaninfo);
	 DelCmd(s_Guardian, "CHANLIST",  cmd_chanlist);
	 DelCmd(s_Guardian, "USERINFO",  cmd_userinfo);
	 DelCmd(s_Guardian, "USERLIST",  cmd_userlist);
	 DelCmd(s_Guardian, "SERVLIST",  cmd_servlist);
         DelCmd(s_Guardian, "EVENTLIST", cmd_eventlist);
	 DelCmd(s_Guardian, "SENDQ",     cmd_sendq);
	 DelCmd(s_Guardian, "FDLIST",    cmd_fdlist);
	 DelCmd(s_Guardian, "SEVENTLIST",cmd_seventlist);
	 DelCmd(s_Guardian, "SERVCMDS",  cmd_servcmds);
	 DelCmd(s_Guardian, "UCMDINFO",  cmd_ucmdinfo);
	 DelCmd(s_Guardian, "CMDINFO",   cmd_usercmds);
	 DelCmd(s_Guardian, "PROGSTATS", cmd_progstats);

        return;


}

/****************************************************/

static void cmd_progstats (User *u, char *argv)
{

	FILE * fp;
	char buf[400];
	char cmd[100];

	sendto_one(s_Guardian,u,"Program CPU Statistics");

	snprintf(cmd,(sizeof(cmd) - 1), "ps ux -p %lu | grep \'security\'",(unsigned long) getpid());
	fp = popen(cmd,"r");

	fread(buf,1,sizeof(buf),fp);

	sendto_one(s_Guardian,u,"%s",buf);

	pclose(fp);

	return;
}



static void cmd_chaninfo	(User *source, char *argv)
{
	dlink_node	*dl;
	char		*params[15];
	Channel		*c;
	struct ChanUser	*cu;
	int			cnt, i;
	char		nicklist[512];

	size_t nicklist_size = 0;

	i = 0;

	if (!source->oper)
	{
		sendto_one (s_Guardian, source, "Access Denied");
		return;
	}

	if (argv == NULL)
	{
		sendto_one (s_Guardian, source, "CHANINFO requires a parameter");
		return;
	}

	cnt	= generictoken (' ', 15, argv, params);

	if (cnt < 1)
	{
		sendto_one (s_Guardian, source, "CHANINFO requires a parameter");
		return;
	}

	if (!(c = find_channel (params[0])))
	{
		sendto_one (s_Guardian, source, "Unknown channel %s", params[0]);
		return;
	}


	memset(nicklist,0,sizeof(nicklist));

	nicklist[0]	= '\0';

	sendto_one (s_Guardian, source, "Information about \002%s\002", c->name);
	sendto_one (s_Guardian, source, " Timestamp: %lu", (unsigned long) c->channelts);
	sendto_one (s_Guardian, source, " Modes: %s", create_chanmode_string(c));
	sendto_one (s_Guardian, source, " Users in channel: ");

	DLINK_FOREACH (dl, c->users.head)
	{
		cu	= dl->data;

		if (!cu->user)
			continue;

		if ((nicklist_size + strlen(cu->user->nick)) >= (sizeof(nicklist) - 1))
		{
			sendto_one(s_Guardian, source, "%s",nicklist);
			memset(nicklist,0,sizeof(nicklist));
			nicklist_size = 0;

			nicklist[0] = '\0';

		}

		if (nicklist[0] != '\0')
		{

			strlcat (nicklist, " ", sizeof(nicklist));
			strlcat (nicklist, cu->user->nick, sizeof(nicklist));
		}
		else
			strlcpy (nicklist, cu->user->nick, sizeof(nicklist));

		nicklist_size += strlen(cu->user->nick);

		i++;

	}

	if (i > 0)
		sendto_one (s_Guardian, source, " Users: %s", nicklist);

	return;
}

static void cmd_userinfo	(User *source, char *argv)
{
	dlink_node	*dl;
	User		*user;
	Channel		*c;
	int			i;
	char		channels[512];

	if (!argv)
	{
		sendto_one (s_Guardian, source, "USERINFO requires a parameter");
		return;
	}

	if (!(user = find_user (argv)))
	{
		sendto_one (s_Guardian, source, "Unknown user \002%s\002", argv);
		return;
	}

	channels[0]	= '\0';

	sendto_one (s_Guardian, source, "Information about \002%s\002 [%s]", user->nick, (HasId(user))? user->uid : "No UID");
	sendto_one (s_Guardian, source, " Userhost: %s@%s", user->user, user->host);
	sendto_one (s_Guardian, source, " IP Addr: %s",(user->ip)? user->ip : "No IP");
	sendto_one (s_Guardian, source, " Realname: %s", user->realname);
	sendto_one (s_Guardian, source, " Server: %s", user->serv->name);
	sendto_one (s_Guardian, source, " Version: %s", (user->version[0] != '\0')? user->version : "No Version Reply");
	sendto_one (s_Guardian, source, " Timestamp: %lu", (unsigned long) user->age);
	sendto_one (s_Guardian, source, " Modes: %s", create_usermode_string(user));

	DLINK_FOREACH (dl, user->channels.head)
	{
		c	= dl->data;

		if (channels[0] != '\0')
		{
			strlcat (channels, " ", sizeof(channels));
			strlcat (channels, c->name, sizeof(channels));
		}
		else
			strlcpy (channels, c->name, sizeof(channels));

		i++;

		if (i == 15)
		{
			sendto_one (s_Guardian, source, " Channels: %s", channels);
			channels[0] = '\0';
			i	= 0;
		}
	}

	if (i > 0)
		sendto_one (s_Guardian, source, " Channels: %s", channels);

	return;
}

static void cmd_userlist	(User *source, char *argv)
{
	User		*user;
	int			i;
	dlink_node	*dl;

	i = 0;

	DLINK_FOREACH (dl, userlist.head)
	{
		user	= dl->data;

		i++;

		sendto_one (s_Guardian, source, "[%d] %s (%s@%s) connected to %s", i, user->nick, user->user,
					user->host, user->serv->name);
	}

	return;
}

static void cmd_chanlist	(User *source, char *argv)
{
	Channel		*c;
	dlink_node	*dl;
	int			i;

	i = 0;

	DLINK_FOREACH (dl, channels.head)
	{
		c	= dl->data;
		i++;

		sendto_one (s_Guardian, source, "[%d] %s containing %d user(s)", i, c->name, c->usercnt);
	}

	return;
}

static void cmd_sendq	(User *source, char *argv)
{
	struct sendq	*sq;
	dlink_node		*dl;
	int				i = 0;

	DLINK_FOREACH (dl, sendq.head)
	{
		sq	= dl->data;
		i++;

		sendto_one (s_Guardian, source, "[%d] buf: %s len: %d pos: %d", i, sq->buf, sq->len, sq->pos);
	}

	return;
}

static void cmd_servlist	(User *source, char *argv)
{
	Link		*li;
	dlink_node	*dl;
	int			i = 0;

	DLINK_FOREACH (dl, servlist.head)
	{
		li	= dl->data;

        i++;

		sendto_one (s_Guardian, source, "[%d] %s connected from %s", i, li->name, (li->uplink)? li->uplink->name : "");
	}

	return;
}

static void cmd_seventlist (User *u, char *av)
{

	dlink_node *dl;
        SockEvent *se;
	int i = 0;
	DLINK_FOREACH(dl,sockevents.head)
	{
		i++;

		se = dl->data;

		sendto_one(s_Guardian,u,"[%d] (%s)",i,se->name);

	}
	return;
}


static void cmd_servcmds    (User *u, char *av)
{

	int i = 0;
	dlink_node *dl;
	ServCmds *sc;
        DLINK_FOREACH(dl,servcmds.head)
        {
			sc = dl->data;

			i++;
			sendto_one(s_Guardian,u,"[%d] (%s) %s",i,sc->name,(sc->module)? sc->module : "Core");

	}

}

static void cmd_ucmdinfo    (User *u, char *av)
{
	dlink_node *dl;
	int index = 0;
	int i = 0;
	UserCmds* uc;

	if (!av)
	{
		sendto_one(s_Guardian,u,"This command requires a paramater");
		return;
	}

	index = HASH(av,1024);
	sendto_one(s_Guardian,u,"Listing USER issued command hooks for (%s)",av);

	DLINK_FOREACH(dl,UserCmdTable[index].head)
	{
		uc = dl->data;

		i++;
		sendto_one(s_Guardian,u,"[%d] %s",i,uc->module);
	}

	return;
}
static void cmd_usercmds    (User *u, char *av)
{


	int i = 0;
	dlink_node *dl;
	int index = HASH(av,1024);
        CmdHash *sc;

	sendto_one(s_Guardian,u,"Listing command hooks for %s",av);

        DLINK_FOREACH(dl,s_Guardian->cmd_list[index].head)
        {
			sc = dl->data;

			i++;
			sendto_one(s_Guardian,u,"[%d] %s",i,(sc->module)? sc->module : "Core");
	}

}

static void cmd_eventlist   (User *source, char *argv)
{
    dlink_node  *dl;
    Events      *ev;
    int         i = 0;

    DLINK_FOREACH (dl, events.head)
    {
        ev  = dl->data;

        i++;

        if (ev->interval == -1)
            sendto_one (s_Guardian, source, "[%d] (%s)", i, ev->name);
        else
            sendto_one (s_Guardian, source, "[%d] (%s) [Next run: %d sec(s)]", i, ev->name, (ev->when - ServerTime));
    }

    if (i <= 0)
        sendto_one (s_Guardian, source, "No events");

    return;
}

static void cmd_fdlist	(User *source, char *argv)
{
	dlink_node		*dl;
	Socket			*s;
	char			type[512], mode[512];

	DLINK_FOREACH (dl, sockets.head)
	{
		s	= dl->data;

		if ((s->flags & SOCK_UPLINK))
			strlcpy (type, "Uplink", sizeof(type));
		else if ((s->flags & SOCK_DCC))
			strlcpy (type, "DCC", sizeof(type));
		else
			strlcpy (type, "OTHER", sizeof(type));

		if ((s->flags & SOCK_READ))
			strlcpy (mode, "Read", sizeof(mode));
		else if ((s->flags & SOCK_WRITE))
			strlcpy (mode, "Write", sizeof(mode));
		else
			strlcpy (mode, "Unknown", sizeof(mode));

		sendto_one (s_Guardian, source, "[%s] [FD: %d] [Type: %s] (%s) (%s)", (s->name[0] != '\0')? s->name : "", s->sd, type, (s->flags & SOCK_CONNECTING)? "Connecting" : "Connected", mode);
	}

	return;
}

