/*
 *         	  OMEGA IRC SECURITY SERVICES
 * 	      	    (C) 2008 Omega Dev Team
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *    $Id: m_omegastats.c 1505 2008-11-28 20:54:10Z twitch $
 */

#include "stdinc.h"
#include "server.h"

#define NAME	"m_omegastats"
#define	AUTHOR	"Jer"
#define VERSION	"$Revision: 1393 $"

#define HTTP_ADDR	"stats.ssnet.ca"
#define HTTP_SERV	"stats.ssnet.ca"
#define HTTP_PORT	80

Socket	*statsock;

int      omegastats_load ();
void     omegastats_close();

static void		sock_read	(Socket *s, int e);
static void		sock_write	(void *);

void	evh_omegastats	(int, char **);

MODHEADER(NAME, VERSION, AUTHOR, MAKE_ABI(0,5,2),omegastats_load, omegastats_close);

int omegastats_load ()
{

	AddTimedEvent	("Omega Statistical Post", evh_omegastats, 900);

	// Always run this at load time
	evh_omegastats (0, NULL);

	return MOD_CONT;
}

void omegastats_close ()
{
	dlink_node	*dl;
	DelTimedEvent	("Omega Statistical Post");

#ifndef HAVE_PTHREAD

	if ((statsock = find_socket ("statsock")))
	{
		close (statsock->sd);
		dl	= dlink_find_delete (statsock, &sockets);
		dlink_free (dl);
		free (statsock);
	}

#endif

	return;
}

static void	sock_read	(Socket *s, int e)
{
	char	*msg;

	while (s->fullLines > 0)
	{
		msg	= GetLine (s);

		if (msg)
			free (msg);
	}

	return;
}

static void sock_write	(void *args)
{
	int		ret;
	char	content[2048];
	char	headers[2048];

	Socket *s = args;

	memset (content, '\0', sizeof(content));
	memset (headers, '\0', sizeof(headers));

	snprintf (content, 2048, "servername=%s&uplink=%s&version=%s&protocol=%s&uplinkname=%s", CfgSettings.servername, CfgSettings.uplink,
				VERSION_STRING_DOTTED, CfgSettings.protocol, (Uplink)? Uplink->name : "not%20connected");
	snprintf (headers, 2048, "User-Agent: Omega/%s m_omegastats.c/%s\nHost: %s\nContent-Length: %d\nContent-Type: application/x-www-form-urlencoded", VERSION_STRING_DOTTED, VERSION, HTTP_SERV, strlen(content));
#if 0
	printf ("Sending:\nPOST /omega/omega.php HTTP/1.1\n%s\n\n%s\r\n\r\n", headers, content);
#endif
	ret = sendto_socket (s, "POST /omega/omega.php HTTP/1.1\n%s\n\n%s\r\n\r\n", headers, content);

	if (ret == 0)
	{
		s->flags	&= ~SOCK_CONNECTING;
		s->flags	&= ~SOCK_WRITE;
		s->flags	|= SOCK_READ;


		return;
	}

	return;
}

void evh_omegastats	(int argc, char **argv)
{
	int					ret;
	struct sockaddr_in	sa;
	dlink_node			*dl;
	char				vhost[63];

	if (!(statsock = find_socket ("statsock")))
	{
	    //new_socket(char*, int, void (*read)(Socket*, int),void (*write)(void*));
		statsock	= new_socket (HTTP_ADDR, HTTP_PORT, (void *) sock_read, sock_write);
		statsock->flags	|= SOCK_CONNECTING|SOCK_WRITE;

		strlcpy (statsock->name, "statsock", sizeof(statsock->name));

        if ((CfgSettings.local_ip[0] == '\0') || match_wild("127.*", CfgSettings.local_ip, 0) ||
            match_wild("192.168.*", CfgSettings.local_ip, 0) || match_wild("10.*", CfgSettings.local_ip, 0))
                            memset (vhost, '\0', sizeof(vhost));
        else
            snprintf (vhost, sizeof(vhost), "%s", CfgSettings.local_ip);

#ifndef HAVE_PTHREAD

        ret = Connect (statsock, HTTP_ADDR, vhost, HTTP_PORT);

        if (!ret)
		{
			dl	= dlink_find_delete (statsock, &sockets);
			dlink_free (dl);
			free (statsock);
			return;
		}
#endif
		return;
	}
	return;
}

